﻿using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using IdeaBlade.EntityModel;

// ReSharper disable InconsistentNaming

namespace CodeFirstWalk.Model.Test
{

    [TestClass]
    public class When_exploring_offline_ProductEntityManager 
    {
        public ProductEntities Manager { get; set; }


        [TestInitialize]
        public void TestInitialize()
        {
            Manager = new ProductEntities(shouldConnect: false);
            Manager.Fetching += (s, e) => Assert.Fail("Manager tried to fetch entities.");
            Manager.Saving += (s, e) => Assert.Fail("Manager tried to save");
        }

        [TestMethod]
        [ExpectedException(typeof(EntityManagerSaveException))]
        public void Fails_if_you_try_to_save()
        {
            Manager.Connect(); // no-no
            Manager.SaveChanges(); // should catch and fail
        }

        [TestMethod]
        public void Can_query_added_supplier()
        {
            var supplier = new Supplier { CompanyName = "Test Supplier" };
            Manager.AddEntity(supplier);
            Assert.AreSame(supplier, Manager.Suppliers.FirstOrDefault());
        }

        [TestMethod]
        public void Then_Supplier_CompanyName_is_uppercased_on_get()
        {
            const string companyName = "lower case name";
            var detachedSupplier = new Supplier { CompanyName = companyName };
            Assert.AreEqual(companyName.ToUpper(), detachedSupplier.CompanyName);
        }

        [TestMethod]
        public void Then_detached_Supplier_CompanyName_is_not_required()
        {
            var supplier = new Supplier { CompanyName = "Supplier" };
            // Manager.AddEntity(supplier); // comment out to keep supplier detached

            supplier.CompanyName = string.Empty;
            var msg = GetFirstValidationErrorMessage(supplier);
            Assert.IsNull(msg, "Did not expect the validation error: " + msg);
        }

        [TestMethod]
        public void Can_validate_detached_Supplier()
        {
            var supplier = new Supplier { CompanyName = "Supplier" };
            // Manager.AddEntity(supplier); // commented out to keep supplier detached

            supplier.CompanyName = string.Empty;

            // var engine = new IdeaBlade.Validation.VerifierEngine(); 
            var engine = Manager.VerifierEngine;
            var validationResults = engine.Execute(supplier);
            var firstErr = validationResults.FirstOrDefault(r => r.IsError);
            Assert.IsNotNull(firstErr, "Expected a validation error");
            var msg = firstErr.Message;
            Assert.IsTrue(msg.Contains("required"),
                          "Error message did not contain 'required'; was " + msg);
        }

        public string GetFirstValidationErrorMessage(IEntity entity)
        {
            var firstError = entity.EntityAspect.ValidationErrors.FirstOrDefault();
            return (null == firstError) ? null : firstError.Message;
        }
        [TestMethod]
        public void Then_attached_Supplier_errs_when_CompanyName_set_StringEmpty()
        {
            var supplier = new Supplier { CompanyName = "Supplier" };
            Manager.AddEntity(supplier);

            supplier.CompanyName = string.Empty;
            var msg = GetFirstValidationErrorMessage(supplier);

            Assert.IsNotNull(msg,
                          "Expected a validation error when CompanyName set to empty string.");
            Assert.IsTrue(msg.Contains("required"),
                          "Error message did not contain 'required'; was " + msg);
        }

        [TestMethod]
        public void Then_attached_Supplier_errs_when_CompanyName_set_null()
        {
            var supplier = new Supplier { CompanyName = "Supplier" };
            Manager.AddEntity(supplier);

            supplier.CompanyName = null;
            var msg = GetFirstValidationErrorMessage(supplier);

            Assert.IsNotNull(msg,
                          "Expected a validation error when CompanyName set to null.");
            Assert.IsTrue(msg.Contains("required"),
                          "Error message did not contain 'required'; was " + msg);
        }

        [TestMethod]
        public void Then_attached_Supplier_errs_when_CompanyName_set_too_long()
        {
            var supplier = new Supplier { CompanyName = "Supplier" };
            Manager.AddEntity(supplier);

            supplier.CompanyName = "Supplier name with more than 20 characters";
            var msg = GetFirstValidationErrorMessage(supplier);

            Assert.IsNotNull(msg,
                   "Expected a validation error when CompanyName > 20 chars.");
            Assert.IsTrue(msg.Contains("20"), // looking for "cannot be longer than 20 character(s)
                   "Error message did not contain '20'; msg = " + msg);
        }

        [TestMethod]
        public void Then_testCategory_is_valid()
        {
            PopulateTestManager(Manager);
            var cat = Manager.Categories.First();

            Assert.AreSame(testCategory, cat,
                "query didn't return the testCategory");
            var state = (cat as IEntity).EntityAspect.EntityState;
            Assert.AreEqual(state, EntityState.Unchanged,
                "unexpected category EntityState, " + state);
            Assert.IsTrue(cat.CategoryId > 0,
                "CategoryId appears to be a temporary id, " + testCategory.CategoryId);
        }

        [TestMethod]
        public void Then_testProduct_is_valid()
        {
            PopulateTestManager(Manager);

            var state = testProduct1.EntityAspect.EntityState;
            Assert.AreEqual(state, EntityState.Unchanged,
                "unexpected testPoduct EntityState, " + state);
            Assert.IsTrue(testProduct1.ProductId > 0,
                "ProductId appears to be a temporary id, " + testProduct1.ProductId);
            Assert.AreSame(testCategory, testProduct1.Category, "unexpected testProduct Category");
            Assert.AreSame(testSupplier, testProduct1.Supplier, "unexpected testProduct Supplier");
        }

        // Add test entities to the Manager
        public void PopulateTestManager(EntityManager manager)
        {
            testSupplier = new Supplier { CompanyName = "Test Supplier" };
            manager.AttachEntity(testSupplier); // attached unchanged as if from query

            testCategory = new Category { CategoryId = 123, CategoryName = "Test Category" };
            manager.AttachEntity(testCategory); // attached unchanged as if from query

            testProduct1 = new Product
            {
                ProductName = "Product 1",
                Category = testCategory,
                Supplier = testSupplier,
                ProductId = 1, // must set after navigation property!
            };
            testProduct2 = new Product
            {
                ProductName = "Product 2",
                Category = testCategory,
                Supplier = testSupplier,
                ProductId = 2, // must set after navigation property!
            };
            // Setting a Product's navigation property pulls it into cache as an ADDED entity
            Manager.AcceptChanges(); // makes everything in cache appear as if queried.

        }

        public Supplier testSupplier;
        public Category testCategory;
        public Product testProduct1;
        public Product testProduct2;

    }
}
