﻿using System;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations.Schema;
using System.Linq;
using IdeaBlade.EntityModel;
using CodeFirstWalk.Model;

namespace CodeFirstWalk
{
    public class MainWindowViewModel : INotifyPropertyChanged
    {
        public MainWindowViewModel()
        {
            Start();
            SetManager();
            AddTestData();
            ShowTestData();
        }
        private void Start()
        {
            Messages = new ObservableCollection<string>();
            Log("Starting application");          
        }

        public ObservableCollection<string> Messages { get; set; }

        private void Log(string message)
        {
            message = String.Format("{0}: {1}", ++_messageCounter, message);
            Console.WriteLine(message);
            Messages.Insert(0, message); // new messages in front
        }
        private int _messageCounter;

        private void SetManager()
        {
            Manager = new ProductEntities();
            Manager.EntityServerError += (s, e) =>
            {
                e.Handled = true; // we're dealing with it here
                Log("Server error: " + e.Exception.Message);
                Manager.RejectChanges(); // undo pending changes
            };
        }

        private ProductEntities Manager { get; set; }

        private void AddTestData()
        {
            Log("Adding test data");
            GetOrAddSupplier();
            AddNewCategory();
            SelectedProduct = AddNewProduct("Chocolate Cream Pie");
            AddNewProduct("Baklava");
            Log("Saving new test data");
            Manager.SaveChanges();
        }

        /// <summary> Get first supplier, if exists, or make one</summary>
        private void GetOrAddSupplier()
        {
            CurrentSupplier = Manager.Suppliers.FirstOrDefault();
            if (null != CurrentSupplier) return;
            CurrentSupplier = new Supplier { CompanyName = "All Fine Foods" };
            Manager.AddEntity(CurrentSupplier);
        }

        private Supplier CurrentSupplier { get; set; }

        private void AddNewCategory()
        {
            CurrentCategory = 
                new Category { CategoryName = "Sweet Things on " + DateTime.Now.ToString("o") };
            Manager.AddEntity(CurrentCategory);
        }
        private Category CurrentCategory { get; set; }

        private Product AddNewProduct(string productName = "A new product")
        {
            var newProduct = new Product
            {
                ProductName = productName,
                Category = CurrentCategory,
                Supplier = CurrentSupplier,
            };

            //Manager.AddEntity(newProduct); // harmless but unnecessary
            return newProduct;
        }

        private void ShowTestData()
        {
            Log("Showing test data");
            var productQuery = Manager.Products.Include("Category");
            Products = new ObservableCollection<Product>(productQuery); // executes & adds to list
            
            // DEMO ONLY - DO NOT DO THIS IN YOUR CODE
            Manager.DefaultQueryStrategy = QueryStrategy.CacheOnly; // let's see only what's in cache
            LogCurrentSupplierProductsCount();
        }

        public ObservableCollection<Product> Products { get; set; }

        private Product _selectedProduct;
        public Product SelectedProduct
        {
            get { return _selectedProduct; }
            set {
                _selectedProduct = value;
                RaisePropertyChanged("SelectedProduct");
            }
        }

        private void LogCurrentSupplierProductsCount()
        {
            Log(
                String.Format("Supplier '{0}' has {1} products.",
                CurrentSupplier.CompanyName, CurrentSupplier.Products.Count));
        }

        public void AddProductAction()
        {
            var newProduct = AddNewProduct();
            Log("Adding new product " + newProduct.ProductId);
            Products.Add(newProduct);
            SelectedProduct = newProduct; 
        }

        public void DeleteProductAction()
        {
            var currentProduct = SelectedProduct;
            if (null == currentProduct)
            {
                Log("No selected product; nothing deleted");
            }
            else
            {
                Log("Deleting product " + currentProduct.ProductId);
                Products.Remove(currentProduct);
                currentProduct.EntityAspect.Delete();
            }
        }

        public void SaveAction()
        {
            Log("Saving changes");
            Manager.SaveChanges();
        }

        public event PropertyChangedEventHandler PropertyChanged;

        private void RaisePropertyChanged(string propertyName)
        {
            var pc = PropertyChanged;
            if (null == pc) return;
            pc(this, new PropertyChangedEventArgs(propertyName));
        }
    }
}
